/*****************************************************
CVAVR V2.05
Chip type               : ATmega8
AVR Core Clock frequency: 8 MHz
Project: automatic soldering station with encoder
*****************************************************/
#include <mega8.h>
#include <delay.h>
#include <stdlib.h>
#include <math.h>

typedef unsigned char   u8;
typedef unsigned int    u16;
typedef signed int      s16;
typedef unsigned long   u32;

#define A   PORTD.4
#define B   PORTD.3
#define C   PORTD.2
#define D   PORTD.1
#define E   PORTD.0
#define F   PORTC.5
#define G   PORTC.4
#define DP    PORTC.3
#define N0    PORTD.6
#define N1    PORTD.5
#define N2    PORTB.7
#define N3    PORTB.6

#define key0 PINC.1
#define key1 PINC.2
#define key2 PINB.3

#define REDL    PORTB.4
#define buzzer  PORTB.5

#define RFB         120000UL        //Ohms
#define RTG         1000            //Ohms
#define K_PAIR           41                   // 1*C = 0.041(mV*1000)
#define ADCMAX          2560000UL       // (mV*1000)
#define K_AMPLIFIER    (1+(RFB/RTG))   // Amplifier coefficient
#define OFFSET      2000            // (mV*1000)
#define MAXTC          (ADCMAX/(K_PAIR*K_AMPLIFIER))*10 //516C*10 

#define timnmbr 5

#define pwm_off do{TCCR1A=0;PORTB.2=PORTB.1=0;}while(0)
#define pwm_on  do{TCCR1A=0b10110000;}while(0)
bit sc_prot_bit;

bit buzzer_on;
enum btn_states {prs, rls, hold} btn0,btn1,btn2;
u8 tkey[timnmbr];
enum dev_condtn {start, norm, sdn, tc_err, ee_err, calibration, sc_prot, timeout_sdn}dev_st;

s16 ambient_t, iron_t, iron_t_def, iron_t_def_tmp;
s16 enc_null;
 
u8  tmr[timnmbr];
u16 tim[timnmbr];

#define msize 10
eeprom u16 ee_iron_tc[msize];
eeprom s16 ee_clb_t_value; 
s16 clb_t_value;

inline void gpio_cfg()
{
 DDRB  = 0b11110111;
 PORTB = 0b00001000;  
 DDRC  = 0b00111000;
 PORTC = 0b01000110;
 DDRD  = 0b01111111;
 PORTD = 0b10000000;  
}
inline void t1_cfg()
{
 #define cpu_speed 8000000UL
 #define pwm_freq  0xffffUL
 ICR1   = cpu_speed/(pwm_freq*2);
 OCR1A  = 0; //ICR1/4; 
 OCR1B  = ICR1-OCR1A;
 TCCR1A = 1<<COM1A1|1<<COM1B1|1<<COM1B0;
 TCCR1B = 1<<WGM13|1<<CS10;   
}
inline void t2_cfg()
{
TCCR2 =0x0A;
OCR2  =(cpu_speed/8)/5000;
TIMSK =1<<OCIE2;
}
inline void adc_cfg()
{
 ADMUX = 1<<REFS1|1<<REFS0;
 ADCSRA = 1<<ADEN|7;
 ADCSRA|= 1<<ADSC;
}
inline void icmp_cfg()
{
 ACSR = 1<<ACBG|1<<ACIE|1<<ACIS1|1<<ACIS0; //int on rising output edge  
}

u8 dbuf[4]={1,2,3,4}, dpp=0;
void convert_data (s16 d)
{
u8 k;
 if(d>=0)
 {
  for (k=0;k<4;k++)
  {
   dbuf[k]=d%10;
   d/=10;
  }
 } 
 else
 {
  d=abs(d);
  for (k=0;k<4;k++)
  {
   dbuf[k]=d%10;
   d/=10;
  }
  dbuf[3]='-'; 
 }
}
void display_data ()
{
  static u8 pc;

  (pc)?(pc--):(pc = 3);
  N0=N1=N2=N3=0; DP=1;

  switch (dbuf[pc])
 {
  case 0: A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 1: A=1; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 2: A=0; B=0; C=1; D=0; E=0; F=1; G=0; break;
  case 3: A=0; B=0; C=0; D=0; E=1; F=1; G=0; break;
  case 4: A=1; B=0; C=0; D=1; E=1; F=0; G=0; break;
  case 5: A=0; B=1; C=0; D=0; E=1; F=0; G=0; break;
  case 6: A=0; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 7: A=0; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 8: A=0; B=0; C=0; D=0; E=0; F=0; G=0; break;
  case 9: A=0; B=0; C=0; D=0; E=1; F=0; G=0; break;
  case ' ': A=1; B=1; C=1; D=1; E=1; F=1; G=1; break;
  case 'u': A=1; B=1; C=0; D=0; E=0; F=1; G=1; break;
  case 'c': A=1; B=1; C=1; D=0; E=0; F=1; G=0; break;
  case 'b': A=1; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 't': A=1; B=1; C=1; D=0; E=0; F=0; G=0; break;
  case 'l': A=1; B=1; C=1; D=0; E=0; F=1; G=1; break;
  case 'o': A=1; B=1; C=0; D=0; E=0; F=1; G=0; break;
  case 'A': A=0; B=0; C=0; D=1; E=0; F=0; G=0; break;
  case 'H': A=1; B=0; C=0; D=1; E=0; F=0; G=0; break;
  case 'C': A=0; B=1; C=1; D=0; E=0; F=0; G=1; break;
  case 'P': A=0; B=0; C=1; D=1; E=0; F=0; G=0; break;
  case 'E': A=0; B=1; C=1; D=0; E=0; F=0; G=0; break;
  case 'U': A=1; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 'F': A=0; B=1; C=1; D=1; E=0; F=0; G=0; break;
  case 'L': A=1; B=1; C=1; D=0; E=0; F=0; G=1; break;
  case 'r': A=1; B=1; C=1; D=1; E=0; F=1; G=0; break;
  case '_': A=1; B=1; C=1; D=0; E=1; F=1; G=1; break;
  case 'd': A=1; B=0; C=0; D=0; E=0; F=1; G=0; break;
  case 'n': A=1; B=1; C=0; D=1; E=0; F=1; G=0; break;
  case 'G': A=0; B=1; C=0; D=0; E=0; F=0; G=1; break;  
  default:  A=1; B=1; C=1; D=1; E=1; F=1; G=0; break; //-
 }

   switch (pc)
 {
  case 0: N0=1; if(dpp == 0) {DP=0;} break;
  case 1: N1=1; if(dpp == 1) {DP=0;} break;
  case 2: N2=1; if(dpp == 2) {DP=0;} break;
  case 3: if(dbuf[3]){N3=1;} if(dpp == 3) {DP=0;} break;
  default: DP=1; break;
 }
 }
void scan_keys ()
{
#define lp  4
#define lh  10
#define lr  4

static u8 count[6];

if ((!key0)&&(!tkey[0])) {count[0]++;tkey[0]=lr;}
  else if((key0)&&(!tkey[0])) {count[0]=count[1]=0;btn0=rls;}
if ((count[0]>lp)&&(btn0!=hold)) {btn0=prs; count[0]=0; count[1]++;}
//if (count[1]>lh) {btn0=hold; count[1]=0;}

if ((!key1)&&(!tkey[1])) {count[2]++;tkey[1]=lr;}
  else if((key1)&&(!tkey[1])) {count[3]=count[2]=0;btn1=rls;}
if ((count[2]>lp)&&(btn1!=hold)) {btn1=prs; count[2]=0; count[3]++;}
//if (count[3]>lh) {btn1=hold; count[3]=0;}

if ((!key2)&&(!tkey[2])) {count[4]++;tkey[2]=lr;}
  else if((key2)&&(!tkey[2])) {count[5]=count[4]=0;btn2=rls;}
if ((count[4]>0xf0)&&(btn2!=hold)) {btn2=prs; count[4]=0; count[5]++;}
if (count[5]>lh) {btn2=hold; count[5]=0;}
}

struct encoder_struct
{
 s16* value;
 s16  val_min;
 s16  val_max;
 u8   step;
 u8   cycle_form;
}encoder_data={&iron_t_def_tmp,-100,100,1,0}; 
void scan_enc (struct encoder_struct sst)
{
u8 New; bit a1,a0;
static u8 EncState;

if(btn0==rls) a0=1; else a0=0;
if(btn1==rls) a1=1; else a1=0; 
New = (a1<<1)+a0;


 
switch(EncState)
{
case 0:
{
if(New == 2) 
  if(*sst.value<sst.val_max-sst.step)*sst.value+=sst.step;
  else if(!sst.cycle_form)*sst.value=sst.val_min;
  else *sst.value=sst.val_max;       
if(New == 1) 
  if(*sst.value>sst.val_min+sst.step)*sst.value-=sst.step;
  else if(!sst.cycle_form)*sst.value=sst.val_max;
  else *sst.value=sst.val_min;
break;
}
case 1:
{
if(New == 0) 
  if(*sst.value<sst.val_max-sst.step)*sst.value+=sst.step;
  else if(!sst.cycle_form)*sst.value=sst.val_min;
  else *sst.value=sst.val_max;  
if(New == 3) 
  if(*sst.value>sst.val_min+sst.step)*sst.value-=sst.step;
  else if(!sst.cycle_form)*sst.value=sst.val_max;
  else *sst.value=sst.val_min; 
break;
}
case 2:
{
if(New == 3) 
  if(*sst.value<sst.val_max-sst.step)*sst.value+=sst.step;
  else if(!sst.cycle_form)*sst.value=sst.val_min;
  else *sst.value=sst.val_max;  
if(New == 0) 
  if(*sst.value>sst.val_min+sst.step)*sst.value-=sst.step;
  else if(!sst.cycle_form)*sst.value=sst.val_max;
  else *sst.value=sst.val_min;
break;
}
case 3:
{
if(New == 1) 
  if(*sst.value<sst.val_max-sst.step)*sst.value+=sst.step;
  else if(!sst.cycle_form)*sst.value=sst.val_min;
  else *sst.value=sst.val_max;  
if(New == 2) 
  if(*sst.value>sst.val_min+sst.step)*sst.value-=sst.step;
  else if(!sst.cycle_form)*sst.value=sst.val_max;
  else *sst.value=sst.val_min;
break;
}
}
 
EncState = New;
}

u16 ADC0_fr;
void scan_adc()
{
#define ADC_itn 16
u8 i;
static u16 ADC0_tmp [ADC_itn]; 

if(!ADCSRA.ADSC)
 {
  for (i=0; i<(ADC_itn-1); i++ ) {ADC0_tmp [i] = ADC0_tmp [(u16)i+1];}
  ADC0_tmp [ADC_itn-1] = ADCW;
  ADC0_fr = 0;
  for (i=0; i<(ADC_itn); i++) {ADC0_fr += ADC0_tmp [i];}
  ADC0_fr /= ADC_itn;
  ADCSRA.ADSC=1; 
 } 
}

u8 DS_int=0;
unsigned char DS18B20_init ()
{
bit OK_Flag;
#asm ("cli")
PORTB &= ~(1 << DS_int); //   
DDRB |= (1 << DS_int);
delay_us(490);
DDRB &= ~(1 << DS_int);
delay_us(68);
OK_Flag = (PINB & (1 << DS_int)); //    
delay_us(422);
#asm ("sei")
return OK_Flag; //  OK_Flag = 0  , OK_Flag = 1   
}
unsigned char read_18b20 ()
{
unsigned char i;   
unsigned char dat = 0;

#asm ("cli")
for(i = 0;i < 8;i++) 
{
DDRB |= (1 << DS_int);
delay_us(2);        
DDRB &= ~(1 << DS_int);
delay_us(4);        
dat = dat >> 1;    
if(PINB & (1 << DS_int))
{
dat |= 0x80; 
}
delay_us(62);
}
#asm ("sei")
return dat;
}
void write_18b20(unsigned char dat)
{
unsigned char i;

#asm ("cli")
for(i = 0;i < 8;i++)
{
DDRB |= (1 << DS_int);
delay_us(2);      
if(dat & 0x01)
{
DDRB &= ~(1 << DS_int);
}
else
{
DDRB |= (1 << DS_int);
}
dat = dat >> 1; 
delay_us(62); 
DDRB &= ~(1 << DS_int);
delay_us(2); 
}
#asm ("sei")
}
signed int read_ds18 ()
{
static bit trg=0; u8 Temp_H, Temp_L;

if(!trg)
{
trg=1;tim[0]=5000;
DS18B20_init();       
write_18b20(0xCC);     //skip rom
write_18b20(0x44);     // run conversion
return 7777;
}
//delay_ms(1000);

else if(!tim[0])
{
trg=0;
DS18B20_init();        
write_18b20(0xCC);     // skip rom
write_18b20(0xBE);     // read spad
Temp_L = read_18b20(); 
Temp_H = read_18b20();

if(Temp_H & (1 << 4))    // t<0
{
signed int tmp;

tmp = ((int)Temp_H << 8) | Temp_L;
tmp = ~tmp +1;
Temp_L = tmp;
Temp_H = tmp >> 8;
return -((((int)Temp_H << 4) & 0b11110000)|(Temp_L >> 4))*10-(((Temp_L & 0b00001111)*625)/1000);
}
else                           //t>=0
{
return ((((int)Temp_H << 4) & 0b11110000)|(Temp_L >> 4))*10+(((Temp_L & 0b00001111)*625)/1000);
}
}
else return 7777;
}

void eetwt (u16 databyte)
{
 u8 i; bit trg;
 
 trg=0;
 for(i=0;i<msize;i++)
 {
    if((ee_iron_tc[i]!=0xffff)&&(ee_iron_tc[i]!=databyte))
    {
     if(i<msize-1)
      {
       ee_iron_tc[(u16)i+1]=databyte;
       ee_iron_tc[i]=0xffff;return;
      } 
     else 
      {
       ee_iron_tc[0]=databyte;
       ee_iron_tc[i]=0xffff;return;
      } 
    }
    else if(ee_iron_tc[i]==databyte) trg=1;
 }
 if(!trg)ee_iron_tc[2]=databyte;  
}
u16 eetrd ()
{
 u8 i;
 for(i=0;i<msize;i++)
  {if(ee_iron_tc[i]!=0xffff)return ee_iron_tc[i];else if(i==msize-1) return 7777;}
}

u8 beep (u16 tmsec)
{
static u16 tmp; static bit trg;

if(!trg){trg=1; tim[1]=5; tmp=tmsec; buzzer_on=1;}
if(!tmp) {trg=buzzer_on=buzzer=0;return 0;}
else if(!tim[1]){tmp--;tim[1]=5; return 1;}
}  

void devcon(); void set_pwm(u8 someval); u8 t_stab_process();

interrupt [TIM2_COMP] void t2_ISR() //5kHz
{
 u8 i;
 
 scan_adc();
 scan_keys ();
 scan_enc (encoder_data);   
 
 for(i=0;i<timnmbr;i++)
 {
  if(tkey[i])tkey[i]--;
  if(tim[i])tim[i]--;
  if(tmr[i])tmr[i]--;
 }
 
 if(buzzer_on){buzzer^=1; beep(0);}
 #asm("wdr")
}
interrupt [ANA_COMP] void acmp()
{
 pwm_off; dev_st=sc_prot; sc_prot_bit=1;
}


void main(void)
{
gpio_cfg();
t1_cfg();
t2_cfg();
adc_cfg();
icmp_cfg();

clb_t_value=ee_clb_t_value;
if(!key2) dev_st=calibration; else dev_st=start;

if      (eetrd()==7777) {eetwt(3000);iron_t_def=iron_t_def_tmp=3000;}
else if (eetrd()>5000)  {dev_st=ee_err;}
else                    {iron_t_def=iron_t_def_tmp=eetrd();}


 
#pragma optsize-
WDTCR=0x18;
WDTCR=0x08;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

#asm("sei")

while (1)
      { 
        display_data();
        devcon();  

      }
}

void devcon()
{
 static bit itr=0, trg_2st=0, btn_trg=1;
 u8 i;
 static u16 timeout;
 
 if(btn2==rls){btn_trg=0;}

switch(dev_st)
{
 case start:
 {
   
  if(!itr)
  {
  itr=1; trg_2st=0; tim[3]=15000; timeout=0xffff;  
  dbuf[3]='t';dbuf[2]='c';dbuf[1]='o';dbuf[0]='o';dpp=2;
  beep(100); pwm_on; for(i=0;i<0xff;i++){set_pwm(i);}
  
  encoder_data.value    = &iron_t_def_tmp;
  encoder_data.val_min  = 1000;
  encoder_data.val_max  = 4500;
  encoder_data.step     = 10;
  encoder_data.cycle_form = 1;
  }
  
  if((tim[3])&&(itr)&&(!trg_2st))
   {
    read_ds18();      
    return;
   } 
  else 
  {
   if(!trg_2st){trg_2st=1;tim[3]=10000;}
   ambient_t=read_ds18();
   if(ambient_t!=7777)
   {
    convert_data(ambient_t/10);dbuf[3]='t';dbuf[2]='c';
    if(!tim[3])
      {
       if(ADC0_fr<1000)
        {
         if(DS18B20_init()){ambient_t=200;} //no pr of DS18
         dev_st=norm;
        }
       else{dev_st=tc_err;}
       
      }

   }
  } 
    
 if(dev_st!=start){itr=0;}      
 }
 break;

 case norm:                 
 {                         
  if(!itr){itr=1; dpp=0;}

  if(!tmr[1])
  {
   tmr[1]=0xff; 
   
   if(timeout)timeout--; else dev_st=timeout_sdn;

    //iron_t=((((u32)ADC0_fr-(((u32)OFFSET*K_AMPLIFIER*0x3ff)/ADCMAX))*MAXTC)/0x3ff)+ambient_t+clb_t_value;
    iron_t=((((u32)ADC0_fr)*MAXTC)/0x3ff)+ambient_t+clb_t_value;
    
   if(tim[4]){convert_data(iron_t_def_tmp/10);} 
   else 
   {
    static u8 ldl;
    if(!ldl){ldl=10;convert_data(iron_t/10);} else ldl--;
    if(iron_t_def!=eetrd()) eetwt(iron_t_def);
   } 
   
   if((iron_t_def_tmp>>3)!=(iron_t_def>>3)){tim[4]=20000; iron_t_def=iron_t_def_tmp; timeout=0xffff;} 
    
   t_stab_process();
  }
    
   
  
 if(btn2==prs)timeout=0xffff;
 if((btn2==hold)&&(!btn_trg)){btn_trg=1; dev_st=sdn;}
 if(dev_st!=norm){itr=0;}  
 }
 break;

 case sdn:
 {
  if(!itr)
  {
  itr=1; set_pwm(0); pwm_off; encoder_data.value = &iron_t_def_tmp;
  dbuf[3]='_';dbuf[2]='_';dbuf[1]='_';dbuf[0]='_';dpp=5;
  } 
  
  if((iron_t_def_tmp>iron_t_def+50)||(iron_t_def_tmp<iron_t_def-50)) dev_st=start; 
  if((btn2==hold)&&(!btn_trg)){btn_trg=1; dev_st=start;}

  if(dev_st!=sdn){itr=0;}    
 }
 break;  
 
 case tc_err:
 {
  static s16 pp, pp_tmp;
  
  if(!itr)
  {
  itr=1;  set_pwm(0); pwm_off; pp=pp_tmp=0; 
  encoder_data.value    = &pp_tmp;
  encoder_data.val_min  = 0;
  encoder_data.val_max  = 0xff;
  encoder_data.step     = 1;
  encoder_data.cycle_form = 1; 
  dbuf[3]='t';dbuf[2]='c';dbuf[1]='E';dbuf[0]='r';dpp=2;
  } 

 if(pp!=pp_tmp)
  {
   pp=pp_tmp; pwm_on;
   convert_data(pp_tmp*100/0xff); 
   dpp=3; dbuf[3]='P';
   set_pwm((u8)pp_tmp);
  }
 
 if((btn2==hold)&&(!btn_trg)){btn_trg=1; dev_st=start;} 
 if(dev_st!=tc_err){itr=0;}   
 } 
 break;
 
 case ee_err:
 {
  
  if(!itr)
  {
  itr=1; set_pwm(0); encoder_data.value= &enc_null; tim[3]=20000;
  dbuf[3]='E';dbuf[2]='E';dbuf[1]='E';dbuf[0]='r';dpp=2; 
  beep(500);
  for(i=0;i<msize;i++){ee_iron_tc[i]=0xffff;} 
  }  
 
 if(!tim[3])
 {
 iron_t_def=iron_t_def_tmp=3000;
 dev_st=start;
 }
 if(dev_st!=ee_err){itr=0;}   
 } 
 break; 

 case sc_prot:
 {
  if(sc_prot_bit){sc_prot_bit=0;itr=0;}
  if(!itr)
  {
   itr=1; set_pwm(0); encoder_data.value= &enc_null;
   dbuf[3]=5;dbuf[2]='c';dbuf[1]='P';dbuf[0]='r';dpp=2; 
   beep(300);
   delay_ms(300);
   beep(300);
   delay_ms(300);
   beep(300);
  }  
 

  if((btn2==hold)&&(!btn_trg)){btn_trg=1; dev_st=start;}
  if(dev_st!=sc_prot){itr=0;}   
 } 
 break;
 
 case timeout_sdn:
 {
  if(!itr)
  {
   itr=1; pwm_off; set_pwm(0); encoder_data.value = &iron_t_def_tmp;
   dbuf[3]='_';dbuf[2]='t';dbuf[1]='o';dbuf[0]='_';dpp=5; 
   beep(1000);
  }  
 
  if((iron_t_def_tmp>iron_t_def+50)||(iron_t_def_tmp<iron_t_def-50)) dev_st=start; 
  if((btn2==hold)&&(!btn_trg)){btn_trg=1; dev_st=start;}
  if(dev_st!=timeout_sdn){itr=0;}   
 } 
 break; 
 
 case calibration:        
 {
 static s16 tmp_clbr_val;
 
  if(!itr)
  {
  itr=1;trg_2st=0; tim[3]=7500; tmp_clbr_val=clb_t_value; //set_pwm(0); 
  dbuf[3]='C';dbuf[2]='L';dbuf[1]='b';dbuf[0]='r';dpp=5;
  beep(500);
  
  encoder_data.value    = &tmp_clbr_val;
  encoder_data.val_min  = -1000;
  encoder_data.val_max  = 1000;
  encoder_data.step     = 10;
  encoder_data.cycle_form = 1;
  } 
  
  if((tim[3])&&(itr)&&(!trg_2st)) return;
  else 
  {
   if(!trg_2st){trg_2st=1;tim[3]=0xffff;}
    
   if(!tmr[0]){tmr[0]=0xff;convert_data(tmp_clbr_val/10);}
   
   if(tmp_clbr_val!=clb_t_value) {tim[3]=0xffff;clb_t_value=tmp_clbr_val;}
   else if(!tim[3])
    {
     if(ee_clb_t_value!=clb_t_value) ee_clb_t_value=clb_t_value; 
     dev_st=start;
    }
    
   if((btn2==hold)&&(!btn_trg))
   {
    btn_trg=1; 
    if(ee_clb_t_value!=clb_t_value) ee_clb_t_value=clb_t_value; 
    dev_st=start;
   } 
  }
  

  
 if(dev_st!=calibration){itr=0;tim[3]=0;} 
 } 
 break; 
     
 default:break; 
}
}

void set_pwm(u8 someval) 
{
#define dtv 5 //*250nS 
u8 tmpA,tmpB;

tmpA=((u16)((ICR1>>1)-dtv)*someval)>>8;
tmpB=ICR1-tmpA;
#asm("cli")  
 OCR1A  = tmpA; 
 OCR1B  = tmpB;
#asm("sei") 
}
u8 t_stab_process()
{
 static u8 pwm_factor;
 
 if (iron_t<(iron_t_def_tmp-(iron_t_def_tmp>>6)))   {pwm_factor=0xff;} 
 else if((iron_t<iron_t_def_tmp)&&(pwm_factor<0xff)){pwm_factor++;}
 else if ((iron_t>iron_t_def_tmp)&&(pwm_factor))    {pwm_factor--;}
 set_pwm(pwm_factor);
 
 if(iron_t>(iron_t_def_tmp-(iron_t_def_tmp>>4))&&(iron_t<(iron_t_def_tmp+(iron_t_def_tmp>>4))))
  REDL=1;
 else REDL=0; 
 return pwm_factor;
}